/*
Copyright (C) 2004  The PARI group.

This file is part of the PARI package.

PARI/GP is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation. It is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY WHATSOEVER.

Check the License for details. You should have received a copy of it, along
with the package; see the file 'COPYING'. If not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include "pari.h"
#include "tree.h"

int linecount;
int newcomment(void)
{
  int n=stack_new(&s_comment);
  comment *c=com+n;
  stack_init(&c->s,sizeof(*c->txt),(void **)&c->txt);
  return n;
}

void pushcomment(int n, char x)
{
  comment *c=com+n;
  int m=stack_new(&c->s);
  c->txt[m]=x;
}

int newnodecom(Ffunc f, int x, int y, int com)
{
  int n=stack_new(&s_node);
  tree[n].f=f;
  tree[n].x=x;
  tree[n].y=y;
  tree[n].t=Gnotype;
  tree[n].m=0;
  tree[n].e=NULL;
  tree[n].lineno=linecount;
  tree[n].comment=com;
  return n;
}
int newnode(Ffunc f, int x, int y)
{
  return newnodecom(f,x,y,-1);
}
int newopcall(OPerator op, int x, int y)
{
  if (y==-1)
    return newnode(Fentryfunc,op,x);
  else
    return newnode(Fentryfunc,op,newnode(Flistarg,x,y));
}

int newstringvalue(const char *s)
{
  int n=stack_new(&s_value);
  value[n].type=CSTstr;
  value[n].val.str=s;
  return n++;
}

int newquotevalue(const char *s)
{
  int n=stack_new(&s_value);
  value[n].type=CSTquote;
  value[n].val.str=s;
  return n++;
}

int newsmallrealvalue(long small)
{
  int n=stack_new(&s_value);
  value[n].type=CSTsmallreal;
  value[n].val.small=small;
  return n++;
}

int newgenvalue(GEN g)
{
  int n=stack_new(&s_value);
  value[n].type=CSTgen;
  value[n].val.gen=g;
  return n++;
}

int newdotvalue(GEN g)
{
  int n=stack_new(&s_value);
  value[n].type=CSTdot;
  value[n].val.gen=g;
  return n++;
}

extern int errno;

/* newxxxnode functions
 * These unctions take a token value as a string and a comment index.
 * They must return a new Fconst or Fsmall node.
 * They must free the string if they do not reference it.
 * the cast free((char *)s) is to remove the const qualifier.
 */
int newgennode(GEN g)
{
  if (typ(g)==t_INT && lg(g)==3 && g[2]>=0)
    return newnode(Fsmall,g[2],-1);
  else
    return newnode(Fconst,newgenvalue(g),-1);
}

int newdotnode(GEN g)
{
  return newnode(Fconst,newdotvalue(g),-1);
}

int is_const(int n, CSTtype t)
{
  int f=tree[n].f;
  int x=tree[n].x;
  return f==Fconst && value[x].type==t;
}

int newentry(const char *s)
{
  return newstringvalue(s);
}

int isfunc(int n, const char *s)
{
  return tree[n].f==Fentryfunc && !strcmp(s,value[tree[n].x].val.str);
}

int newmember(const char *s)
{
  char *p;
  int n;
  p=calloc(sizeof(*s),3+strlen(s));
  sprintf(p,"_.%s",s);
  n=newentry(p);
  free((char *)s);
  return n;
}
