/*
Copyright (C) 2000-2003  The PARI group.

This file is part of the GP2C package.

PARI/GP is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation. It is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY WHATSOEVER.

Check the License for details. You should have received a copy of it, along
with the package; see the file 'COPYING'. If not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */


#include "stack.h"
int debug;
char *nameparse, *namelib;
int currfunc;
int lastpass;
int autogc;
int warn;

#define STACKSZ (1024)
#define GNIL    (0)
#define GNOARG  (1)
typedef enum {Gvoid, Gnegbool, Gbool, Gsmall, Gint, Greal, Gmp, Glg, Gvecsmall,
Gvec, Gvar, Gpol, Ggen, Ggenstr, Gstr, Gbptr, Ggptr, Gulong, Gtyp, Gnf, Gbnf, Gbnr, Gell, Gbell, Gclgp, Gprid, Ggal, Gnbtype, Gnotype, Ggc} Gtype;

typedef struct
{
  char *txt;
  long lineno;
} error_string;

typedef struct
{
  stack s;
  char *txt;
} comment;

typedef struct
{
  char *s;
  int c;
} strcom;
typedef enum {Fseq,
	      Fmatrix,FmatrixL,FmatrixR,
	      Faffect,
	      Ffacteurmat,
	      Fmatrixelts,Fmatrixlines,
	      Femptyvec,Femptymat,Fmat,
	      Flistarg,

	      Fconst,Fsmall,Fgnil,
	      Frefarg,Ftag,
	      Fentry,Fentryfunc,Fdeffunc,

	      /*This node is generated by genblock, not by parser*/
	      Fblock
} Ffunc;
#define FneedENTRY (Fconst)
#define Flastfunc  (Fblock)

/*see parse.y for definition.
Fblock x: block number y:seq
Fgnil   : it's a noop, but if evaluated leads to 0.
*/
typedef struct node_s
{
  Ffunc f;          /*node function*/
  int x;            /*node left child*/
  int y;            /*node right child*/
  Gtype t;          /*node type*/
  int m;            /*node mode, Mmode bitmask*/
  int lineno;       /*linenumber of corresponding code*/
  int comment;
  entree *e;
} node;

typedef enum {CSTsmall, CSTsmallreal, CSTstr, CSTquote, CSTgen, CSTdot} CSTtype;

typedef struct
{
  CSTtype type;     /*type of data*/
  union value_u
  {
    long small;
    const char *str;
    GEN gen;
  } val;
} value_t;

typedef enum {OPor, OPand, OPeq, OPne, OPge, OPg, OPle, OPl, OPs, OPp, OPsl, OPsr, OPmod, OPdr, OPeuc, OPd, OPm, OPpow, OPcat, OPss, OPpp, OPse ,OPpe ,OPsle ,OPsre ,OPmode ,OPdre ,OPeuce ,OPde ,OPme, OPpl, OPn, OPnb, OPfact, OPderiv, OPtrans, OPnboperator} OPerator;

node *tree;
value_t *value;
comment *com;
error_string *errors;

stack s_node,s_value,s_func,s_ctx,s_aff,s_bloc,s_comment,s_errors;

typedef enum { err_func=-1,err_parse=-2,err_desc=-3 } err_name;

#ifdef __GNUC__
/*Avoid spurious warning*/
void die(int n, char *format, ...) __attribute__ ((noreturn));
#else
void die(int n, char *format, ...);
#endif
void warning(int n, char *format, ...);

int listtostack(int n, int f, int *stack, int nbmax, const char *error,int nerr);
int listcalltostack(int n, int fnum, int *stack, int nbmax, const char *error,int nerr);
int listtostackparent(int n, int f, int *stack, int nbmax, const char *error,int nerr);
int genlistargs(int n,int *stack,int min,int max);
int genlistcats(int y,int *stack,int max);

int newcomment(void);
void pushcomment(int n, char x);
void gencomment(FILE *fout, int n, int flag);

typedef enum {PPend,PPstd,PPdefault,PPdefaultmulti,PPmultiarg,PPsep} PPproto;

PPproto parseproto(char const **q, char *c);

int newnode(Ffunc f, int x, int y);
int newnodecom(Ffunc f, int x, int y, int com);
int newdotnode(GEN g);
int newrealnode(const char *s, int c);
int newintnode(const char *s, int c);
int newgennode(GEN g);
int newdotvalue(GEN g);
int newnumvalue(const char *s);
int newgenvalue(GEN g);
int newsmallrealvalue(long small);
int newstringvalue(const char *s);
int newquotevalue(const char *s);
int newentry(const char *s);
int newmember(const char *s);
int newleaf(int n);
int newopcall(OPerator s, int x, int y);

